package TJournal::Data;

# Data.pm -- author: Oliver Bossert
# Copyright (C) 2004 Oliver Bossert
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

use strict;
use XML::Simple;
use Data::Dumper;

use TJournal::Conf;

############################################################
sub new { # Generate a new Object
############################################################
  my $type = shift;
  my $class = ref($type) || $type;
  my $self = {};
  bless $self, $class;

  TJournal::Conf::default("Data::Basedir", ".");

  $self->{'file'} = shift || TJournal::Conf::get("Data::Basedir") . "/data.xml";
  $self->readfile();

  return $self;
}

############################################################
sub readfile { # Read the data
############################################################
  my $self = shift;
  my $filename = shift;
  $self->{'file'} = $filename if ( defined($filename) );
  my $xs = new XML::Simple(KeyAttr => [ 'depot' ],
			   ForceArray => [ 'depot' ]);
  $self->{'data'} = $xs->XMLin($self->{'file'}) 
    unless (! -r $self->{'file'});
}

############################################################
sub writefile {
############################################################
  my $self = shift;
  my $xs = new XML::Simple( XMLDecl => 1,
			    KeyAttr => [ 'text' ] );
  my $string = $xs->XMLout($self->{'data'});
  open CONF, ">" . $self->{'file'} or die
    "Could not write in file: " . $self->{'file'};
  print CONF $string;
  close CONF;
}

############################################################
sub syncwithdepot {
############################################################
  my $self = shift;
  my $depotname = shift;

  require Depot::ReadConfig;
  my $config = Depot::ReadConfig->new();
  $config->get();

  my $depot = $config->{'depots'}->{$depotname}->{'depot'};
  if ( defined(@{$depot->{'history'}->{'trade'}}) ) {
    foreach my $pos ( sort {$a->{'open'}->{'date'} cmp $b->{'open'}->{'date'}} @{$depot->{'history'}->{'trade'}} ) {

      # Check if the Position is already in the Journal
      my $exists = 0;
      if (defined($self->{'data'}->{'trade'})) {
	foreach my $trade ( @{$self->{'data'}->{'trade'}} ) {
	  if ($trade->{'source'} == $depotname &&
	      $trade->{'source_id'} == $pos->{'id'} ) {
	    $exists = 1;
	  }
	}
      }
      if ( $exists == 0 ) {
	$pos->{'source'} = $depotname;
	$pos->{'source_id'} = $pos->{'id'};
	$pos->{'id'} = $self->newid();
	$pos->{'text'}->[0] = "=head2 Kommentare\n\n";
	foreach my $c ( @{$pos->{'comment'}} ) {
	  $pos->{'text'}->[0] .= $c->{'id'} . ":\n";
	  $pos->{'text'}->[0] .= $c->{'value'} . "\n\n";
	}
	$pos->{'text'}->[1] = " ";
	push @{$self->{'data'}->{'trade'}}, $pos;
      }
    }
  }
}

############################################################
sub newid {
############################################################
  my $self = shift;
  my $id = 0;
  if (defined($self->{'data'}->{'trade'})) {
    foreach my $trade ( @{$self->{'data'}->{'trade'}} ) {
      $id = $trade->{'id'} if ( defined($trade->{'id'}) &&
				$trade->{'id'} > $id);
    }
  }
  $id++;
  return $id;
}



1;




