package TJournal::GUI;

# GUI.pm -- author: Oliver Bossert
# Copyright (C) 2004 Oliver Bossert
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

use strict;

use TJournal::Conf;
use TJournal::DB;

use Tk;
use Tk::ToolBar;
use Tk::NoteBook;
use Tk::TableMatrix;
use Tk::Adjuster;
use Tk::ROText;
BEGIN {
  unless (eval "use Tk::PNG") {
    warn "Could not load PNG-module...";
  }
}

#use GT::Conf;
#use GT::Eval;

############################################################
sub new { # Generate a new Object
############################################################
  my $type = shift;
  my $class = ref($type) || $type;
  my $self = {};
  bless $self, $class;

  $self->{'data'} = shift;

# Enable this for the use with GT:
#  GT::Conf::load();
#  $self->{'db'} = create_standard_object("DB::" . GT::Conf::get("DB::module"));

  $self->{'db'} = TJournal::DB->new();

  return $self;
}

############################################################
sub display { # Do all the GUI-stuff
############################################################
  my $self = shift;
  $self->{'mw'} = MainWindow->new(-title=>"Olf's Trading Journal");

  # Create the Toolbar
  $self->create_toolbar();

  # Create NoteBook
  $self->create_notebook();

  # Create Overview-Table
  $self->create_table_overview();

  # Create Details-Page
  $self->create_detail_page();

  # Create Analysis-Page
  $self->create_analysis_trade_page();

  # Create Analysis-Page
  $self->create_analysis_portfolio_page();

  MainLoop;
}


############################################################
sub create_notebook {
############################################################
  my $self = shift;

  $self->{'notebook'} = $self->{'mw'}->NoteBook()->pack( -expand => 1,
							 -fill => 'both' );

  $self->{'nb_overview'} = $self->{'notebook'}->add("overview", 
						    -label => "Overview" );

  $self->{'nb_details'} = $self->{'notebook'}->add("details", 
						    -label => "Details",
						   -raisecmd => sub { $self->update_detail_page } );

  $self->{'nb_analysis_trade'} = $self->{'notebook'}->add("analysis_trade", 
							  -label => "Analysis Trade",
							  -raisecmd => sub { $self->update_analysis_trade_page() }  );

  $self->{'nb_analysis_portfolio'} = $self->{'notebook'}->add("analysis_portfolio", 
							      -label => "Analysis Portfolio",
							      -raisecmd => sub { $self->update_portfolio_trade_page() }  );

}


############################################################
sub create_toolbar {
############################################################
  my $self = shift;
  $self->{'toolbar'} = $self->{'mw'}->ToolBar(qw/-movable 1 -side top/ );
#					      -indicatorcolor blue/);

  $self->{'toolbar'}->ToolButton(-image   => 'fileopen22',
				 -tip     => 'Open File',
				 -command => sub {
				   my @types =
				     (["XML Datei",           [qw/.xml/]],
				      ["All files",                '*']
				     );
				   my $file = $self->{'mw'}->getOpenFile(-filetypes => \@types,
									 -initialfile => 'data.xml');
				   $self->{'data'}->readfile( $file );
				   $self->update_table();
				   $self->{'table'}->selectionSet("1,1","1,1");
				   $self->update_detail_page();
				 }
				);

  $self->{'toolbar'}->ToolButton(-image   => 'appsheet16',
				 -tip     => 'Sync with DepotXML',
				 -command => sub {
				   my @types =
				     (["XML Datei",           [qw/.xml/]],
				      ["All files",                '*']
				     );
				   my $file = $self->{'mw'}->getOpenFile(-filetypes => \@types,
									 -initialfile => 'depot-000001.xml');

				   my $dir = "./";
				   if ( $file =~ /(.*\/)([^\/]+$)/ ) {
				     $dir = $1;
				     $file = $2;
				    # print "$dir --> $file\n";
				   }

				   require Depot::Conf;
				   Depot::Conf::load();
				   Depot::Conf::default("Config::Basedir", $dir);

				   #print $file . "\n";
				   $self->{'data'}->syncwithdepot( $file );

				   $self->update_table();
				   $self->{'table'}->selectionSet("1,1","1,1");
				   $self->update_detail_page();
				 }
				);

  $self->{'toolbar'}->ToolButton(-image   => 'filenew22',
				 -tip     => 'New Trade',
				 -command => sub {
				   $self->new_record();
				 }
				);

  $self->{'toolbar'}->ToolButton(-image   => 'filesave22',
				 -tip     => 'Save Trade',
				 -command => sub {
				   $self->save_record();
				 }
				);

  $self->{'toolbar'}->ToolButton(-image   => 'actreload22',
				 -tip     => 'Render Text',
				 -command => sub {
				   $self->render_text();
				 }
				);

  $self->{'toolbar'}->ToolButton(-image   => 'actredo22',
				 -tip     => 'Export',
				 -command => sub {
				   my @types =
				     (["Text Datei",           [qw/.txt/]],
				      #["Tabelle CVS",          [qw/.cvs/]],
				      ["All files",                '*']
				     );
				   my $file = $self->{'mw'}->getSaveFile(-filetypes => \@types,
									 -initialfile => 'Untitled',
									 -defaultextension => '.txt');
				   my ($row, $col) = $self->update_table();
				   open OUT, ">$file" or die "Could not write to $file...";
				   for my $r (0..$row) {
				     for my $c (0..$col+1) {
				       print OUT $self->{'table-data'}->{"$r,$c"} . "\t";
				     }
				     print OUT "\n";
				   }
				   close OUT;
				 }
				);
}


############################################################
sub create_table_overview {
############################################################
  my $self = shift;
  my $row = 1;
  my $col = 8;

  ($row, $col) = $self->update_table();

  $self->{'table'} = $self->{'nb_overview'}->Scrolled('TableMatrix',
						      -justify => 'right',
						      -rows => $row,
						      -cols => $col+1,
						      -width => 6,
						      -height => $row,
						      -titlerows => 1,
						      -titlecols => 0,
						      -variable => $self->{'table-data'},
						      -roworigin =>  0,
						      -colorigin  => 0,
						      -colstretchmode => 'all',
						      -rowstretchmode => 'all',
						      -selectmode => 'browse',
						      -state => "disabled",
						      -sparsearray => 0,
						      -wrap => 1,
						      -multiline => 1,
						      -colwidth => 7,
						      -rowheight => 2,
						      -selcmd => sub {
							#print STDERR "SELECTED\n";
							#print STDERR join("\n", @_) . "\n";
							$self->{'table'}->tagDelete("sel2");
							$self->{'table'}->tagConfigure('sel2', -bg => 'orange', -relief => 'sunken');
							my @idx = $self->{'table'}->curselection();
							if ( $idx[0] =~ /(\d+),(\d+)/) {
							  #print STDERR "TAGGING: $1 \n";
							  $self->{'table'}->tagRow("sel2", $1);
							}
							return 0;
						      },
						      -rowtagcommand => sub { 
							my $row = shift;
							return "plus" if ($self->{'table-data'}->{"$row,8"} > 0);
							return "minus";
						      }
						     )->pack(-expand => 1,
							     -fill=>'both',
							     -anchor => "nw");

  $self->{'table'}->tagConfigure('plus', -fg => 'darkgreen', -relief => 'sunken');
  $self->{'table'}->tagConfigure('minus', -fg => 'darkred', -relief => 'sunken');

  $self->{'table'}->tagConfigure('active', -bg => 'gray90', -relief => 'sunken');
  $self->{'table'}->tagConfigure( 'title', -bg => 'gray85', -fg => 'black', -relief => 'sunken');

  $self->{'table'}->tagConfigure('sel', -bg => 'orange', -relief => 'sunken');

  $self->{'table'}->colWidth(0, 20);

}


############################################################
sub create_detail_page {
############################################################
  my $self = shift;
  my ($l, $e);

  # Adjuster
  $self->{'nb_details_left'} = $self->{'nb_details'}->Frame->pack( -side => "left",
								   -fill=>'y',
								   -expand => 1 );

  $self->{'nb_details_adj1'} = $self->{'nb_details'}->Adjuster( -widget => $self->{'nb_details_left'},
								-side => "left")->pack(-side=>"left",
										       -fill=>"y");
  $self->{'nb_details_right'} = $self->{'nb_details'}->Frame->pack( -side=>'bottom',
								    -fill=>'both',
								    -expand=>1);


  $self->{'nb_details_top'} = $self->{'nb_details_right'}->Frame->pack( -side => "top",
									-fill=>'x',
									-expand => 1 );

  $self->{'nb_details_adj2'} = $self->{'nb_details_right'}->Adjuster( -widget => $self->{'nb_details_top'},
								      -side => "top")->pack(-side=>"top",
											    -fill=>"x");
  $self->{'nb_details_bot'} = $self->{'nb_details_right'}->Frame->pack( -side=>'bottom',
									-fill=>'both',
									-expand=>1);


  $self->{'nb_details_left_top'} = $self->{'nb_details_left'}->Frame->pack( -side=>"top" );

  $self->{'details'}->{'name'} = $self->{'nb_details_left_top'}->Label(-text => "NAME" );
  Tk::grid($self->{'details'}->{'name'}, "-");

  $l = $self->{'nb_details_left_top'}->Label(-text => "WKN" );
  $self->{'details'}->{'wkn'} = $self->{'nb_details_left_top'}->Entry(-text => "WKN" );
  Tk::grid($l,$self->{'details'}->{'wkn'});

  $l = $self->{'nb_details_left_top'}->Label(-text => "Type" );
  $self->{'details'}->{'type'} = $self->{'nb_details_left_top'}->Entry(-text => "long");
  Tk::grid($l,$self->{'details'}->{'type'});

  $l = $self->{'nb_details_left_top'}->Label(-text => "Number:" );
  $self->{'details'}->{'nb'} = $self->{'nb_details_left_top'}->Entry(-text => "0" );
  Tk::grid($l,$self->{'details'}->{'nb'});

  $l = $self->{'nb_details_left_top'}->Label(-text => "Open Date:" );
  $self->{'details'}->{'opendate'} = $self->{'nb_details_left_top'}->Entry(-text => "Open Date" );
  Tk::grid($l,$self->{'details'}->{'opendate'});

  $l = $self->{'nb_details_left_top'}->Label(-text => "Open Price:" );
  $self->{'details'}->{'openprice'} = $self->{'nb_details_left_top'}->Entry(-text => "Open Price" );
  Tk::grid($l,$self->{'details'}->{'openprice'});

  $l = $self->{'nb_details_left_top'}->Label(-text => "Close Date:" );
  $self->{'details'}->{'closedate'} = $self->{'nb_details_left_top'}->Entry(-text => "Close Date" );
  Tk::grid($l,$self->{'details'}->{'closedate'});

  $l = $self->{'nb_details_left_top'}->Label(-text => "Close Price:" );
  $self->{'details'}->{'closeprice'} = $self->{'nb_details_left_top'}->Entry(-text => "Close Price" );
  Tk::grid($l,$self->{'details'}->{'closeprice'});

  $l = $self->{'nb_details_left_top'}->Label(-text => "Fee:" );
  $self->{'details'}->{'fee'} = $self->{'nb_details_left_top'}->Entry(-text => "Fee" );
  Tk::grid($l,$self->{'details'}->{'fee'});

  my $FONT = '-*-Helvetica-Medium-R-Normal--*-140-*-*-*-*-*-*';
  $self->{'text_output'} =  $self->{'nb_details_top'}->Scrolled('ROText',
								-font       => $FONT,
								-scrollbars => 'e',
								-wrap       => 'word')->pack(-expand => 1,
											     -fill => "both");

  $self->{'text_input'} =  $self->{'nb_details_bot'}->Scrolled('Text',
							       -scrollbars => 'e',
							       -wrap       => 'word')->pack(-expand => 1,
											    -fill => "both");

}

############################################################
sub update_detail_page {
############################################################
  my $self = shift;

  my $id = 0;
  my @idx = $self->{'table'}->curselection();
  if ( $idx[0] =~ /(\d+),(\d+)/) {
    $id = $self->{'table-data'}->{"$1,idx"};
  }
  $self->{'curid'} = $id;

  my $trade;
  if (defined($self->{'data'}->{'data'}->{'trade'})) {
    foreach $trade ( @{$self->{'data'}->{'data'}->{'trade'}} ) {
      if ($trade->{'id'} == $id) {

	# Name
	$self->{'details'}->{'name'}->configure(-text => $self->{'db'}->get_db_name( $trade->{'wkn'} ) );

	# Wkn
	$self->{'details'}->{'wkn'}->delete(0,"end");
	$self->{'details'}->{'wkn'}->insert("end", $trade->{'wkn'});

	# Type
	$self->{'details'}->{'type'}->delete(0,"end");
	$self->{'details'}->{'type'}->insert("end", $trade->{'type'});

	# Number
	$self->{'details'}->{'nb'}->delete(0,"end");
	$self->{'details'}->{'nb'}->insert("end", $trade->{'nb'});

	# Open Date
	$self->{'details'}->{'opendate'}->delete(0,"end");
	$self->{'details'}->{'opendate'}->insert("end", $trade->{'open'}->{'date'});

	# Open Price
	$self->{'details'}->{'openprice'}->delete(0,"end");
	$self->{'details'}->{'openprice'}->insert("end", $trade->{'open'}->{'value'});

	# Close Date
	$self->{'details'}->{'closedate'}->delete(0,"end");
	$self->{'details'}->{'closedate'}->insert("end", $trade->{'close'}->{'date'});

	# Close Price
	$self->{'details'}->{'closeprice'}->delete(0,"end");
	$self->{'details'}->{'closeprice'}->insert("end", $trade->{'close'}->{'value'});

	# Fee
	$self->{'details'}->{'fee'}->delete(0,"end");
	$self->{'details'}->{'fee'}->insert("end", $trade->{'fee'}->{'value'});

	# Text
	$self->{'text_input'}->delete("1.0", "end");
	if ( ref($trade->{'text'}) =~ /ARRAY/ ) {
	  foreach my $line (@{$trade->{'text'}}) {
	    $self->{'text_input'}->insert("end", $line) 
	      unless (ref($line) =~/HASH/ );
	  }
	} else {
	  $self->{'text_input'}->insert("end", $trade->{'text'} );
	}

	# Render the Text
	$self->render_text();

	last;
      }
    }
  }

}

############################################################
sub save_record {
############################################################
  my $self = shift;

  my $trade;
  if (defined($self->{'data'}->{'data'}->{'trade'})) {
    foreach $trade ( @{$self->{'data'}->{'data'}->{'trade'}} ) {
      if ($trade->{'id'} == $self->{'curid'}) {

	# Wkn
	$trade->{'wkn'} = $self->{'details'}->{'wkn'}->get();

	# Type
	$trade->{'type'} = $self->{'details'}->{'type'}->get();

	# Number
	$trade->{'nb'} = $self->{'details'}->{'nb'}->get();

	# Open Date
	$trade->{'open'}->{'date'} = $self->{'details'}->{'opendate'}->get();

	# Open Price
	$trade->{'open'}->{'value'} = $self->{'details'}->{'openprice'}->get();

	# Close Date
	$trade->{'close'}->{'date'} = $self->{'details'}->{'closedate'}->get();

	# Close Price
	$trade->{'close'}->{'value'} = $self->{'details'}->{'closeprice'}->get();

	# Fee
	$trade->{'fee'}->{'value'} = $self->{'details'}->{'fee'}->get();

	# Text
	@{$trade->{'text'}} = ();
	$trade->{'text'}->[0] = $self->{'text_input'}->get("1.0", "end");
	$trade->{'text'}->[1] = " ";

	last;
      }
    }
  }

  $self->update_table();

}


############################################################
sub new_record {
############################################################
  my $self = shift;

  my $trade;
  $trade->{'wkn'} = "";
  $trade->{'id'} = $self->{'data'}->newid();
  $trade->{'type'} = "long";
  $trade->{'nb'} = "0";
  $trade->{'open'}->{'date'} = timestamp();
  $trade->{'open'}->{'value'} = "0";
  $trade->{'close'}->{'date'} = timestamp();
  $trade->{'close'}->{'value'} = "0";
  $trade->{'fee'}->{'value'} = "0";
  @{$trade->{'text'}} = ();
  $trade->{'text'}->[0] = "Bemerkungen:";
  $trade->{'text'}->[1] = " ";

  push @{$self->{'data'}->{'data'}->{'trade'}}, $trade;

  $self->update_table();
  $self->{'table'}->selectionSet("1,1","1,1");
  $self->update_detail_page();

}


############################################################
sub render_text {
############################################################
  my $self = shift;
  my $content = $self->{'text_input'}->get("1.0", "end");

  $self->{'text_output'}->delete("1.0", "end");
  $self->{'text_output'}->tagConfigure(qw/head1 -font -*-Helvetica-Bold-R-Normal--*-180-*-*-*-*-*-*/);
  $self->{'text_output'}->tagConfigure(qw/head2 -font -*-Helvetica-Bold-R-Normal--*-160-*-*-*-*-*-*/);

  foreach my $line ( split /\n/s, $content ) {
    if ( $line =~ /^=(\w+)\s+(.*)/ ) {
      my $tag = $1;
      my $content = $2;
      if ($tag eq "img") {
	my $image = $self->{'mw'}->Photo( -file => $content );
	if ( $image->width > $self->{'text_output'}->width |
	     $image->height > $self->{'text_output'}->height ) {
	  my $factor = $self->{'text_output'}->width / $image->width;
	  $factor = $self->{'text_output'}->height / $image->height 
	    if ($self->{'text_output'}->height / $image->height  < $factor);
	  my $subs = $self->{'mw'}->Photo;
	  my $intfactor = int( 1 / $factor );
	  $intfactor++ if ($intfactor < (1/$factor));
	  $intfactor = 1 if ($intfactor < 1);
	  $subs->copy($image, -subsample => ($intfactor, $intfactor) );
	  $image->blank;
	  $image->copy($subs);
	  $subs->delete();
	}
	$self->{'text_output'}->imageCreate("end", -image => $image ) 
	  unless ( ! -r $content );
      } else {
	$self->{'text_output'}->insert("end", $content ."\n", $tag);
      }
    } else {
      $self->{'text_output'}->insert("end", $line ."\n");
    }
  }
}

############################################################
sub update_table {
############################################################
  my $self = shift;
  my $row = 1;
  my $col = 8;

  $self->{'table-data'}->{"0,0"} = "WKN";
  $self->{'table-data'}->{"0,1"} = "Type";
  $self->{'table-data'}->{"0,2"} = "Nb";
  $self->{'table-data'}->{"0,3"} = "OpenDate";
  $self->{'table-data'}->{"0,4"} = "OpenPrice";
  $self->{'table-data'}->{"0,5"} = "CloseDate";
  $self->{'table-data'}->{"0,6"} = "ClosePrice";
  $self->{'table-data'}->{"0,7"} = "Fee";
  $self->{'table-data'}->{"0,8"} = "Win/Loss";

  if (defined($self->{'data'}->{'data'}->{'trade'})) {
    foreach my $trade ( sort {$b->{'open'}->{'date'} cmp $a->{'open'}->{'date'}} 
			@{$self->{'data'}->{'data'}->{'trade'}} ) {

      $self->{'table-data'}->{"$row,0"} = $trade->{'wkn'};
      $self->{'table-data'}->{"$row,1"} = $trade->{'type'};
      $self->{'table-data'}->{"$row,2"} = $trade->{'nb'};
      $self->{'table-data'}->{"$row,3"} = $trade->{'open'}->{'date'};
      $self->{'table-data'}->{"$row,3"} =~ s/_/ /;
      $self->{'table-data'}->{"$row,4"} = $trade->{'open'}->{'value'};
      $self->{'table-data'}->{"$row,5"} = $trade->{'close'}->{'date'};
      $self->{'table-data'}->{"$row,5"} =~ s/_/ /;
      $self->{'table-data'}->{"$row,6"} = $trade->{'close'}->{'value'};
      $self->{'table-data'}->{"$row,7"} = $trade->{'fee'}->{'value'};

      my $prozent;
      if ( defined( $trade->{'fee'}->{'value'} ) ) {
	$prozent = ( 100 * ( ( $trade->{'nb'} * $trade->{'close'}->{'value'} ) - 
			     ( $trade->{'nb'} * $trade->{'open'}->{'value'} ) -
			     $trade->{'fee'}->{'value'} ) ) / 
			       ($trade->{'nb'}*$trade->{'open'}->{'value'})
				   unless( ($trade->{'nb'}*$trade->{'open'}->{'value'})==0);
	if ($trade->{'type'} =~ /short/i) {
	  $prozent = ( -100 * ( ( $trade->{'nb'} * $trade->{'close'}->{'value'} ) - 
				( $trade->{'nb'} * $trade->{'open'}->{'value'} ) +
				$trade->{'fee'}->{'value'} ) ) / 
				  ($trade->{'nb'}*$trade->{'open'}->{'value'})
				    unless( ($trade->{'nb'}*$trade->{'open'}->{'value'})==0);
	}
      }
      $self->{'table-data'}->{"$row,8"} = sprintf("%.2f",$prozent);
      $self->{'table-data'}->{"$row,8"} = "+" . $self->{'table-data'}->{"$row,8"} if ($prozent > 0);

      $self->{'table-data'}->{"$row,idx"} = $trade->{'id'};

      $row++;
    }
  }

  if ( defined( $self->{'table'} ) ) {
    $self->{'table'}->configure(-rows => $row,
				-cols => $col+1 );
  }

  return ($row, $col);
}


############################################################
sub create_analysis_trade_page {
############################################################
  my $self = shift;

  $self->{'nb_analysis_trade_top'} = 
    $self->{'nb_analysis_trade'}->Frame->pack( -side=>"top",
					     -anchor=>"nw" );

  my $l;

  $self->{'analysis_trade'}->{'name'} = $self->{'nb_analysis_trade_top'}->Label(-text => "NAME" );
  Tk::grid($self->{'analysis_trade'}->{'name'}, "-",-sticky=>'w');

  $l = $self->{'nb_analysis_trade_top'}->Label(-text => "WKN" );
  $self->{'analysis_trade'}->{'wkn'} = $self->{'nb_analysis_trade_top'}->Label(-text => "WKN" );
  Tk::grid($l,$self->{'analysis_trade'}->{'wkn'},-sticky=>'w');

  $l = $self->{'nb_analysis_trade_top'}->Label(-text => "Type" );
  $self->{'analysis_trade'}->{'type'} = $self->{'nb_analysis_trade_top'}->Label(-text => "long");
  Tk::grid($l,$self->{'analysis_trade'}->{'type'},-sticky=>'w');

  $l = $self->{'nb_analysis_trade_top'}->Label(-text => "Number:" );
  $self->{'analysis_trade'}->{'nb'} = $self->{'nb_analysis_trade_top'}->Label(-text => "0" );
  Tk::grid($l,$self->{'analysis_trade'}->{'nb'},-sticky=>'w');

  $l = $self->{'nb_analysis_trade_top'}->Label(-text => "Open Date:" );
  $self->{'analysis_trade'}->{'opendate'} = $self->{'nb_analysis_trade_top'}->Label(-text => "Open Date" );
  Tk::grid($l,$self->{'analysis_trade'}->{'opendate'},-sticky=>'w');

  $l = $self->{'nb_analysis_trade_top'}->Label(-text => "Open Price:" );
  $self->{'analysis_trade'}->{'openprice'} = $self->{'nb_analysis_trade_top'}->Label(-text => "Open Price" );
  Tk::grid($l,$self->{'analysis_trade'}->{'openprice'},-sticky=>'w');

  $l = $self->{'nb_analysis_trade_top'}->Label(-text => "Close Date:" );
  $self->{'analysis_trade'}->{'closedate'} = $self->{'nb_analysis_trade_top'}->Label(-text => "Close Date" );
  Tk::grid($l,$self->{'analysis_trade'}->{'closedate'},-sticky=>'w');

  $l = $self->{'nb_analysis_trade_top'}->Label(-text => "Close Price:" );
  $self->{'analysis_trade'}->{'closeprice'} = $self->{'nb_analysis_trade_top'}->Label(-text => "Close Price" );
  Tk::grid($l,$self->{'analysis_trade'}->{'closeprice'},-sticky=>'w');

  $l = $self->{'nb_analysis_trade_top'}->Label(-text => "Fee:" );
  $self->{'analysis_trade'}->{'fee'} = $self->{'nb_analysis_trade_top'}->Label(-text => "Fee" );
  Tk::grid($l,$self->{'analysis_trade'}->{'fee'},-sticky=>'w');

  $l = $self->{'nb_analysis_trade_top'}->Label(-text => "" );
  Tk::grid($l,$l);

  $l = $self->{'nb_analysis_trade_top'}->Label(-text => "Percentage:" );
  $self->{'analysis_trade'}->{'percentage'} = $self->{'nb_analysis_trade_top'}->Label(-text => "Percentage" );
  Tk::grid($l,$self->{'analysis_trade'}->{'percentage'},-sticky=>'w');

  $l = $self->{'nb_analysis_trade_top'}->Label(-text => "Net:" );
  $self->{'analysis_trade'}->{'net'} = $self->{'nb_analysis_trade_top'}->Label(-text => "Net" );
  Tk::grid($l,$self->{'analysis_trade'}->{'net'},-sticky=>'w');



}


############################################################
sub update_analysis_trade_page {
############################################################
  my $self = shift;

  my $id = 0;
  my @idx = $self->{'table'}->curselection();
  if ( $idx[0] =~ /(\d+),(\d+)/) {
    $id = $self->{'table-data'}->{"$1,idx"};
  }
  $self->{'curid'} = $id;

  my $trade;
  if (defined($self->{'data'}->{'data'}->{'trade'})) {
    foreach $trade ( @{$self->{'data'}->{'data'}->{'trade'}} ) {
      if ($trade->{'id'} == $id) {

	# Name
	$self->{'analysis_trade'}->{'name'}->configure(-text => $self->{'db'}->get_db_name( $trade->{'wkn'} ) );

	# Wkn
	$self->{'analysis_trade'}->{'wkn'}->configure(-text => $trade->{'wkn'});

	# Type
	$self->{'analysis_trade'}->{'type'}->configure(-text => $trade->{'type'});

	# Number
	$self->{'analysis_trade'}->{'nb'}->configure(-text => $trade->{'nb'});

	# Open Date
	$self->{'analysis_trade'}->{'opendate'}->configure(-text => $trade->{'open'}->{'date'});

	# Open Price
	$self->{'analysis_trade'}->{'openprice'}->configure(-text => $trade->{'open'}->{'value'});

	# Close Date
	$self->{'analysis_trade'}->{'closedate'}->configure(-text => $trade->{'close'}->{'date'});

	# Close Price
	$self->{'analysis_trade'}->{'closeprice'}->configure(-text => $trade->{'close'}->{'value'});

	# Fee
	$self->{'analysis_trade'}->{'fee'}->configure(-text => $trade->{'fee'}->{'value'});

	# Percentage
	my $prozent;
	if ( defined( $trade->{'fee'}->{'value'} ) ) {
	  $prozent = ( 100 * ( ( $trade->{'nb'} * $trade->{'close'}->{'value'} ) - 
			       ( $trade->{'nb'} * $trade->{'open'}->{'value'} ) -
			       $trade->{'fee'}->{'value'} ) ) / 
				 ($trade->{'nb'}*$trade->{'open'}->{'value'})
				   unless( ($trade->{'nb'}*$trade->{'open'}->{'value'})==0);
	  if ($trade->{'type'} =~ /short/i) {
	    $prozent = ( -100 * ( ( $trade->{'nb'} * $trade->{'close'}->{'value'} ) - 
				  ( $trade->{'nb'} * $trade->{'open'}->{'value'} ) +
				  $trade->{'fee'}->{'value'} ) ) / 
				    ($trade->{'nb'}*$trade->{'open'}->{'value'})
				      unless( ($trade->{'nb'}*$trade->{'open'}->{'value'})==0);
	  }
	}

	$prozent = sprintf("%.2f",$prozent);
	$prozent = "+" . $prozent if ($prozent > 0);
	$prozent .= "%";
	$self->{'analysis_trade'}->{'percentage'}->configure(-text => $prozent );
	
	# Net Gain / Loss:
	my $net;
	if ( defined( $trade->{'fee'}->{'value'} ) ) {
	  $net =  ( $trade->{'nb'} * $trade->{'close'}->{'value'} ) - 
	    ( $trade->{'nb'} * $trade->{'open'}->{'value'} ) -
	      $trade->{'fee'}->{'value'};
	  if ($trade->{'type'} =~ /short/i) {
	    $net = ( $trade->{'nb'} * $trade->{'close'}->{'value'} ) - 
	      ( $trade->{'nb'} * $trade->{'open'}->{'value'} ) +
		$trade->{'fee'}->{'value'};
	  }
	}
	$net = sprintf("%.2f",$net);
	$net = "+" . $net if ($net > 0);
	$self->{'analysis_trade'}->{'net'}->configure(-text => $net );

	last;
      }
    }
  }

}


############################################################
sub create_analysis_portfolio_page {
############################################################
  my $self = shift;

  $self->{'nb_analysis_portfolio_top'} = 
    $self->{'nb_analysis_portfolio'}->Frame->pack( -side=>"top",
						-anchor=>"nw" );
  my $l;

  $l = $self->{'nb_analysis_portfolio_top'}->Label(-text => "Portfolio Analyse" );
  Tk::grid($l, "-",-sticky=>'w');

  $l = $self->{'nb_analysis_portfolio_top'}->Label(-text => "Number Winning Trades: " );
  $self->{'portfolio_trade'}->{'nb_win'} = $self->{'nb_analysis_portfolio_top'}->Label(-text => "0" );
  Tk::grid($l,$self->{'portfolio_trade'}->{'nb_win'},-sticky=>'w');

  $l = $self->{'nb_analysis_portfolio_top'}->Label(-text => "Number Loosing Trades: " );
  $self->{'portfolio_trade'}->{'nb_loss'} = $self->{'nb_analysis_portfolio_top'}->Label(-text => "0" );
  Tk::grid($l,$self->{'portfolio_trade'}->{'nb_loss'},-sticky=>'w');

  $l = $self->{'nb_analysis_portfolio_top'}->Label(-text => "Win/Loss-Ratio: " );
  $self->{'portfolio_trade'}->{'win_loss'} = $self->{'nb_analysis_portfolio_top'}->Label(-text => "0" );
  Tk::grid($l,$self->{'portfolio_trade'}->{'win_loss'},-sticky=>'w');

}


############################################################
sub update_portfolio_trade_page {
############################################################
  my $self = shift;

  my $nbwin = 0;
  my $nbloss = 0;

  my $trade;
  if (defined($self->{'data'}->{'data'}->{'trade'})) {
    foreach $trade ( @{$self->{'data'}->{'data'}->{'trade'}} ) {

      # Percentage
	my $prozent;
	if ( defined( $trade->{'fee'}->{'value'} ) ) {
	  $prozent = ( 100 * ( ( $trade->{'nb'} * $trade->{'close'}->{'value'} ) - 
			       ( $trade->{'nb'} * $trade->{'open'}->{'value'} ) -
			       $trade->{'fee'}->{'value'} ) ) / 
				 ($trade->{'nb'}*$trade->{'open'}->{'value'})
				   unless( ($trade->{'nb'}*$trade->{'open'}->{'value'})==0);
	  if ($trade->{'type'} =~ /short/i) {
	    $prozent = ( -100 * ( ( $trade->{'nb'} * $trade->{'close'}->{'value'} ) - 
				  ( $trade->{'nb'} * $trade->{'open'}->{'value'} ) +
				  $trade->{'fee'}->{'value'} ) ) / 
				    ($trade->{'nb'}*$trade->{'open'}->{'value'})
				      unless( ($trade->{'nb'}*$trade->{'open'}->{'value'})==0);
	  }
	}

	# Net Gain / Loss:
	my $net;
	if ( defined( $trade->{'fee'}->{'value'} ) ) {
	  $net =  ( $trade->{'nb'} * $trade->{'close'}->{'value'} ) - 
	    ( $trade->{'nb'} * $trade->{'open'}->{'value'} ) -
	      $trade->{'fee'}->{'value'};
	  if ($trade->{'type'} =~ /short/i) {
	    $net = ( $trade->{'nb'} * $trade->{'close'}->{'value'} ) - 
	      ( $trade->{'nb'} * $trade->{'open'}->{'value'} ) +
		$trade->{'fee'}->{'value'};
	  }
	}

	$nbwin++ if ( $prozent >= 0);
	$nbloss++ if ( $prozent < 0);

      }
  }

  $self->{'portfolio_trade'}->{'nb_win'}->configure( -text=> $nbwin );
  $self->{'portfolio_trade'}->{'nb_loss'}->configure( -text=> $nbloss );
  my $winratio = 100 * $nbwin / ($nbloss+$nbwin);
  $winratio =  sprintf("%.2f",$winratio);
  $winratio .= " %";
  $self->{'portfolio_trade'}->{'win_loss'}->configure( -text=> $winratio );

}



############################################################
sub timestamp { # Creates a timestamp
############################################################
  my @t = localtime(time);
  $t[5] += 1900;
  $t[4] += 1;
  my $date = sprintf("%04d-%02d-%02d_%02d:%02d:%02d", $t[5], $t[4], $t[3], $t[2], $t[1], $t[0]);
  return $date;
}


1;
